@@ -30,7 +30,7 @@
 extern "C" {
 #endif
 
-#define LIBTASN1_VERSION "0.2.18"
+#define LIBTASN1_VERSION "0.3.0"
 
 #include <sys/types.h>
 #include <time.h>
@@ -77,11 +77,11 @@ typedef int asn1_retCode;  /* type returned by libtasn1 functions */
 /*****************************************/
 /* Constants returned by asn1_read_tag   */
 /*****************************************/
-#define ASN1_CLASS_UNIVERSAL        1
-#define ASN1_CLASS_APPLICATION      2
-#define ASN1_CLASS_CONTEXT_SPECIFIC 3
-#define ASN1_CLASS_PRIVATE          4
-
+#define ASN1_CLASS_UNIVERSAL        0x00  /* old: 1 */
+#define ASN1_CLASS_APPLICATION      0x40  /* old: 2 */
+#define ASN1_CLASS_CONTEXT_SPECIFIC 0x80  /* old: 3 */
+#define ASN1_CLASS_PRIVATE          0xC0  /* old: 4 */
+#define ASN1_CLASS_STRUCTURED       0x20
 
 /*****************************************/
 /* Constants returned by asn1_read_tag   */
@@ -193,6 +193,31 @@ const char* libtasn1_strerror(asn1_retCode error);
 
 void libtasn1_perror(asn1_retCode error);
 
+/* DER utility functions. */
+
+  int asn1_get_tag_der(const unsigned char *der, int der_len,
+		       unsigned char *class,int  *len, unsigned long *tag);
+
+  void asn1_octet_der(const unsigned char *str,int str_len,
+		      unsigned char *der,int *der_len);
+
+  asn1_retCode asn1_get_octet_der(const unsigned char *der, int der_len,
+				  int *ret_len,unsigned char *str,
+				  int str_size, int *str_len);
+
+  void asn1_bit_der(const unsigned char *str,int bit_len,
+		    unsigned char *der,int *der_len);
+
+  asn1_retCode asn1_get_bit_der(const unsigned char *der, int der_len,
+				int *ret_len,unsigned char *str,
+				int str_size, int *bit_len);
+
+  signed long asn1_get_length_der(const unsigned char *der, int der_len,
+				  int *len);
+
+  void asn1_length_der(unsigned long len,unsigned char *ans, int *ans_len);
+
+
 #ifdef __cplusplus
 }
 #endif
